/* 
 * Adium is the legal property of its developers, whose names are listed in the copyright file included
 * with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//#import <AIImageAdditions.h>
//#import <Adium/AIUserIcons.h>
//#import <AISharedAdium.h>
//#import <AIControllerProtocol.h>
#import <AIListContact.h>

#import "AIStudiVZAccount.h"
#import "AIStudiVZAccountViewController.h"
#import "StudiVZProfileFetcher.h"
#import "ABStudiVZPlugin.h"


@implementation AIStudiVZAccount

+ (id) getSVZAccount
{
	return ourself;
}

- (id) init {
    self = [super init];
	ourself = self;
	
	NSNumber *num = [self preferenceForKey:KEY_SVZ_ACCOUNT_TYPE group:GROUP_ACCOUNT_STATUS];
	int item = 0;
	if (num != nil)
		item = [num intValue];

	accountType = item;
//	AILog(@"StudiVZPlugin init: setting accountType to: %@ ", accountType );

	
    return self;
}

- (NSString *) SVZAccountType
{
	NSNumber *num = [self preferenceForKey:KEY_SVZ_ACCOUNT_TYPE group:GROUP_ACCOUNT_STATUS];
	int item = 0;
	if (num != nil)
		item = [num intValue];

	return [NSString stringWithString:( (item == 0) ? @"s" : @"m")];
}

- (NSString *) SVZGroupName
{
	NSNumber *num = [self preferenceForKey:KEY_SVZ_ACCOUNT_TYPE group:GROUP_ACCOUNT_STATUS];
	int item = 0;
	if (num != nil)
		item = [num intValue];

	return [NSString stringWithString:( (item == 0) ? @"StudiVZ" : @"MeinVZ")];
}


- (NSString *)serverSuffix
{
	return @"@nimbuzz.com";
}

/*!
 * @brief Allow a file transfer with an object?
 *
 */
- (BOOL)allowFileTransferWithListObject:(AIListObject *)inListObject
{
	return NO;
}


- (void)configurePurpleAccount
{
	[super configurePurpleAccount];
	
	// force connect server
	purple_account_set_string(account, "connect_server", "snow.nimbuzz.com");
}

- (BOOL)allowAccountUnregistrationIfSupportedByLibpurple
{
	return NO;
}

/*
- (NSString *)unknownGroupName {
    return (AILocalizedString(@"StudiVZ","StudiVZ Plauderkasten"));
}
*/

- (const char *)purpleStatusIDForStatus:(AIStatus *)statusState
							  arguments:(NSMutableDictionary *)arguments
{
	if ([statusState statusType] == AIOfflineStatusType) {
		return "offline";
	} else {
		return "available";
	}
}



- (AIService *)_serviceForUID:(NSString *)contactUID
{

	AIService	*contactService;
	NSString	*contactServiceID = nil;

	if ([contactUID hasSuffix:@"@studivz.nimbuzz.com"]) {
		contactServiceID = @"adium-jabber-studivz";
		contactService = [ABStudiVZPlugin getSVZService];

		#if DEBUG_BUILD
//		AILog( @"StudiVZPlugin: request for service for UID '%@', returning: %@", contactUID, [contactService description] );
		#endif

		if (contactService != nil)
			return contactService;
	}
	
	return [super _serviceForUID:contactUID];
	
}


#pragma mark Contacts


- (void)gotGroupForContact:(AIListContact *)listContact {
	#if DEBUG_BUILD
	AILog(@"StudiVZPlugin: got group for contact: %@ ", [listContact description] );
	#endif
	
	if ([[listContact UID] hasSuffix:@"@studivz.nimbuzz.com"]) {
		#if DEBUG_BUILD
		AILog(@"StudiVZPlugin: setting group for contact: %@ to '%@'", [listContact description], [self SVZGroupName] );
		#endif

		[listContact setRemoteGroupName: [self SVZGroupName]];
	}
	
}


/*!
 * @brief Called when a buddy signs on
 * We use it to get the user icon and set the nick
 */
- (void)updateSignon:(AIListContact *)theContact withData:(void *)data
{
	[super updateSignon:theContact withData:data];

	#if DEBUG_BUILD
	NSLog([@"StudiVZPlugin: Buddy signed on: " stringByAppendingString:[theContact description] ]);
	AILog( @"StudiVZPlugin: Buddy signed on: %@", [theContact description] );
	#endif


	StudiVZProfileFetcher* svpf = [[StudiVZProfileFetcher alloc] initWithContact:theContact andAccount:self];
	[svpf fetchProfile];
	
}




@end
