/* 
 * Adium is the legal property of its developers, whose names are listed in the copyright file included
 * with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#import <Adium/AISharedAdium.h>
#import <Adium/AIControllerProtocol.h>
#import <Adium/AIAdiumProtocol.h>
#import <Adium/AIStatusControllerProtocol.h>
#import <Adium/AIcontactControllerProtocol.h>

//#import <AIImageAdditions.h>
#import <AIUtilities/AIFileManagerAdditions.h>
#import <Adium/AIUserIcons.h>
#import <Adium/AIPlugin.h>
#import <Adium/AIListContact.h>

#import "AIStudiVZAccount.h"
#import "AIStudiVZAccountViewController.h"
#import "StudiVZProfileFetcher.h"
#import "ABStudiVZPlugin.h"

@implementation AIStudiVZAccount

+ (id) getSVZAccount
{
	return ourself;
}

- (id) init {
    self = [super init];
	ourself = self;
	
	NSNumber *num = [self preferenceForKey:KEY_SVZ_ACCOUNT_TYPE group:GROUP_ACCOUNT_STATUS];
	int item = 0;
	if (num != nil)
		item = [num intValue];

	accountType = item;
//	AILogWithSignature(@"StudiVZPlugin init: setting accountType to: %@ ", accountType );

	
    return self;
}

- (int) SVZAccountNum
{
	NSNumber *num = [self preferenceForKey:KEY_SVZ_ACCOUNT_TYPE group:GROUP_ACCOUNT_STATUS];
	int item = 0;
	if (num != nil)
		item = [num intValue];

	return item;
}

- (NSString *) SVZAccountType
{
	return [NSString stringWithString:( [self SVZAccountNum] ? @"m" : @"s")];
}

- (NSString *) SVZLongAccountType
{
	return [NSString stringWithString:( [self SVZAccountNum] ? @"mein" : @"studi")];
}

- (NSString *) SVZGroupName
{
	return [NSString stringWithString:( [self SVZAccountNum] ? @"MeinVZ" : @"StudiVZ")];
}


- (NSString *)serverSuffix
{
	return @"@nimbuzz.com";
}

- (NSString *)webProfileStringForContact:(AIListContact *)contact
{
	return [NSString stringWithFormat:NSLocalizedString(@"%@s StudiVZ Profil öffnen", nil), 
			contact.displayName];
}


/*!
 * @brief Allow a file transfer with an object?
 *
 */
- (BOOL)allowFileTransferWithListObject:(AIListObject *)inListObject
{
	return NO;
}


- (void)configurePurpleAccount
{
	[super configurePurpleAccount];
	
	// force connect server
	purple_account_set_string(account, "connect_server", "snow.nimbuzz.com");
}

- (BOOL)allowAccountUnregistrationIfSupportedByLibpurple
{
	return NO;
}

/*
- (NSString *)unknownGroupName {
    return (AILocalizedString(@"StudiVZ","StudiVZ Plauderkasten"));
}
*/

- (const char *)purpleStatusIDForStatus:(AIStatus *)statusState
							  arguments:(NSMutableDictionary *)arguments
{
	if ([statusState statusType] == AIOfflineStatusType) {
		return "offline";
	} else {
		return "available";
	}
}

/*!
 * @brief We don't manage our own cache, so enable Adiums internal cache.
 */
- (BOOL)managesOwnContactIconCache
{
	return NO;
}


- (AIService *)_serviceForUID:(NSString *)contactUID
{

	AIService	*contactService;
	NSString	*contactServiceID = nil;

	if ([contactUID hasSuffix:@"@studivz.nimbuzz.com"]) {
		contactServiceID = @"adium-jabber-studivz";
		contactService = [ABStudiVZPlugin getSVZService];

		#if DEBUG_BUILD
//		AILogWithSignature( @"StudiVZPlugin: request for service for UID '%@', returning: %@", contactUID, [contactService description] );
		#endif

		if (contactService != nil)
			return contactService;
	}
	
	return [super _serviceForUID:contactUID];
	
}

/*!
 * @brief Menu items for the account's actions
 *
 * Returns an array of menu items for account-specific actions.  This is the best place to add protocol-specific
 * actions that aren't otherwise supported by Adium.  It will only be queried if the account is online.
 * @return NSArray of NSMenuItem instances for this account
 */
- (NSArray *)accountActionMenuItems {

	AILogWithSignature(@"Getting accountActionMenuItems for %@",self);
	NSMutableArray *menu = [[NSMutableArray alloc] init];


	//add the menu items from our super-account, the Jabber imp
    NSArray *supermenu = [super accountActionMenuItems];
    if(supermenu) {
		[menu addObjectsFromArray:supermenu];
        [menu addObject:[NSMenuItem separatorItem]];
	}

	//and now make the new menu item
	
	NSMenuItem *fethAllMenuItem = [[NSMenuItem alloc] initWithTitle:AILocalizedString(@"Refresh all Profiles",nil)
																	action:@selector(FetchAllProfiles:) 
															 keyEquivalent:@""];
	[fethAllMenuItem setTarget:self];
	[menu addObject:fethAllMenuItem];
	[fethAllMenuItem release];

    return [menu autorelease];	

}


#pragma mark Contacts


- (void)gotGroupForContact:(AIListContact *)listContact {
	#if DEBUG_BUILD
		AILogWithSignature(@"[SVZ] StudiVZPlugin: got group for contact: %@ ", [listContact description] );
	#endif
	
	if ([[listContact UID] hasSuffix:@"@studivz.nimbuzz.com"]) {
		#if DEBUG_BUILD
			AILogWithSignature(@"[SVZ] StudiVZPlugin: setting group for contact: %@ to '%@'", [listContact description], [self SVZGroupName] );
		#endif
		//get rid of the annoying 'buddies' group
		[listContact removeRemoteGroupName: @"Buddies"];
		
		[listContact addRemoteGroupName: [self SVZGroupName]];
		
		
		//and check if we are to refresh pics/statuses, fetch profile from web
		NSNumber *num = [self preferenceForKey:KEY_SVZ_FETCH_FREQ group:GROUP_ACCOUNT_STATUS];
		int item = DEF_FETCH_FREQ;
		if (num != nil)
			item = [num intValue];

	
		//fetch the profile if its set to uber-always
		if (item == 0){
			#if DEBUG_BUILD
				AILogWithSignature(@"[SVZ] StudiVZPlugin: refreshing profile for contact: %@", [listContact description] );
			#endif
			StudiVZProfileFetcher* svpf = [[[StudiVZProfileFetcher alloc] initWithContact:listContact andAccount:self] retain];
			[svpf fetchProfile];
		}else{
			#if DEBUG_BUILD
				AILogWithSignature(@"[SVZ] StudiVZPlugin: setting formatted UID of contact: %@", [listContact description] );
			#endif
			//if not then just set the formatted UID, to get rid of the %studivz.net@@studivz.nimbuzz.com postfix
			[listContact setFormattedUID:[[listContact UID] substringToIndex:16] notify:NotifyNow];
		}
	}
}


/*!
 * @brief Called when a buddy signs on
 * We use it to get the user icon and set the nick
 */
- (void)updateSignon:(AIListContact *)theContact withData:(void *)data
{
	[super updateSignon:theContact withData:data];

	#if DEBUG_BUILD
		AILogWithSignature( @"[SVZ] StudiVZPlugin: Buddy signed on: %@", [theContact description] );
	#endif

	//and check if we should update the icon/status, ie. fetch the profile fresh
	NSNumber *num = [self preferenceForKey:KEY_SVZ_FETCH_FREQ group:GROUP_ACCOUNT_STATUS];
	int item = DEF_FETCH_FREQ;
	if (num != nil)
		item = [num intValue];

	int chance = rand()%100;
	
	// #TODO check the mod date of the cached image if its >specific timespan
	//make the ref date (4 weeks into the past)
	
	bool pic_is_stale = false;
	
	NSDate *ref_date = [NSDate dateWithTimeIntervalSinceNow:-CACHED_STALE_SPAN_SECS];
	NSString *path = [[adium cachesPath] stringByAppendingPathComponent:theContact.internalObjectID];
	
	if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
		NSDate *time = [[[NSFileManager defaultManager] attributesOfItemAtPath:path error:NULL] fileModificationDate];
		if ( [time earlierDate:ref_date] == time){
			pic_is_stale = true;
			#if DEBUG_BUILD
				AILogWithSignature( @"[SVZ] StudiVZPlugin: Buddy '%@'s cached pic is stale (>4 weeks), timestamp of pic: %@", [theContact description], [time description]);
			#endif
			
		}
	}else{
		pic_is_stale = true;
		#if DEBUG_BUILD
			AILogWithSignature( @"[SVZ] StudiVZPlugin: Buddy '%@'s cached pic does not exist, refreshing!", [theContact description]);
		#endif
	}
	
	//fetch the profile if its set to always or sometimes and its greater than the set freq
	if (item == 0 || item == 1 || ( item == 2 && chance > FREQ_OF_NEW_MOON) || (item == 3 && pic_is_stale) ){
		StudiVZProfileFetcher* svpf = [[[StudiVZProfileFetcher alloc] initWithContact:theContact andAccount:self] retain];
		[svpf fetchProfile];
	}else			//meaning: never
		//just notify that something has changed
		[theContact notifyOfChangedPropertiesSilently:YES];

}


#pragma mark selectors


/*!
 * @brief Menu action to refresh all profiles
 */
- (IBAction)FetchAllProfiles:(id)sender {



	AIService *svz_service = [ABStudiVZPlugin getSVZService];
	NSMutableArray *contacts;
	NSArray *mcontacts;
	

	AIListGroup *svzgroup = [[adium contactController] groupWithUID:@"StudiVZ"];
	AIListGroup *mvzgroup = [[adium contactController] groupWithUID:@"MeinVZ"];

	contacts = [[adium contactController] allContactsInObject:svzgroup onAccount:(AIAccount *)self];
	mcontacts = [[adium contactController] allContactsInObject:mvzgroup onAccount:(AIAccount *)self]; 
	[contacts addObjectsFromArray:mcontacts];
	 

	#if DEBUG_BUILD
		AILogWithSignature( @"[SVZ] StudiVZPlugin: Refreshing all profiles w/ service: %@ and UID: %@. Got %i contacts.", svz_service, @"studivz.nimbuzz.com", [contacts count]);
	#endif
	

	for (AIAccount *contact in contacts) {

		StudiVZProfileFetcher* svpf = [[[StudiVZProfileFetcher alloc] initWithContact:contact andAccount:self] retain];
		[svpf fetchProfile];
	
	}
}


@end
