/* 
 * Adium is the legal property of its developers, whose names are listed in the copyright file included
 * with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#import <AIStudiVZAccountViewController.h>


@implementation AIStudiVZAccountViewController

- (NSString *)nibName{
    return @"AIStudiVZAccountView";
}


- (void)awakeFromNib
{
	[super awakeFromNib];
	[checkBox_checkMail setEnabled:NO];
}

- (void)configureForAccount:(AIAccount *)inAccount
{

	#if DEBUG_BUILD
	AILogWithSignature(@"[SVZ] StudiVZPlugin configuring account");
	#endif

	[super configureForAccount:inAccount];
	
	ourAccount = inAccount;
	
	//SVZ account
	NSString *SVZ_accountUID = [inAccount preferenceForKey:KEY_SVZ_ACCOUNT_NAME group:GROUP_ACCOUNT_STATUS];
	[textField_SVZ_accountUID setStringValue:(SVZ_accountUID ? SVZ_accountUID : @"")];
	//and pass
	NSString *SVZ_account_password = [inAccount preferenceForKey:KEY_SVZ_ACCOUNT_PASS group:GROUP_ACCOUNT_STATUS];
	[textField_SVZ_password setStringValue:(SVZ_account_password ? SVZ_account_password : @"")];
	
	//type
	NSNumber *num = [inAccount preferenceForKey:KEY_SVZ_ACCOUNT_TYPE group:GROUP_ACCOUNT_STATUS];
	int item = 0;
	
	if (num != nil)
		item = [num intValue];
		
	[comboBox_accountType selectItemAtIndex: item]; 
	[comboBox_accountType setObjectValue:[comboBox_accountType objectValueOfSelectedItem]];

	//fetch freq
	//#TODO check if we can get the latest modif date from the actual icon which is cached on disc, to make it "if older than 4 weeks" or so
	
	num = [inAccount preferenceForKey:KEY_SVZ_FETCH_FREQ group:GROUP_ACCOUNT_STATUS];
	item = DEF_FETCH_FREQ;
	
	if (num != nil)
		item = [num intValue];
		
	[comboBox_ProfFetchFreq selectItemAtIndex: item]; 
	[comboBox_ProfFetchFreq setObjectValue:[comboBox_ProfFetchFreq objectValueOfSelectedItem]];

}

//Save controls
- (void)saveConfiguration
{
    [super saveConfiguration];
	
	//SVZ account
	[ourAccount setPreference:([[textField_SVZ_accountUID stringValue] length] ? [textField_SVZ_accountUID stringValue] : nil)
					forKey:KEY_SVZ_ACCOUNT_NAME group:GROUP_ACCOUNT_STATUS];

	[ourAccount setPreference:([[textField_SVZ_password stringValue] length] ? [textField_SVZ_password stringValue] : nil)
					forKey:KEY_SVZ_ACCOUNT_PASS group:GROUP_ACCOUNT_STATUS];

	[ourAccount setPreference: [NSNumber numberWithInt:[comboBox_accountType indexOfSelectedItem]]
					forKey:KEY_SVZ_ACCOUNT_TYPE group:GROUP_ACCOUNT_STATUS];

	[ourAccount setPreference: [NSNumber numberWithInt:[comboBox_ProfFetchFreq indexOfSelectedItem]]
					forKey:KEY_SVZ_FETCH_FREQ group:GROUP_ACCOUNT_STATUS];

	#if DEBUG_BUILD
	AILogWithSignature(@"[SVZ] StudiVZPlugin saving fetch freq to %i", [comboBox_ProfFetchFreq indexOfSelectedItem]);
	#endif

}

@end
