/* 
 * Adium is the legal property of its developers, whose names are listed in the copyright file included
 * with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#define AISVZService

#import <Adium/AIAdiumProtocol.h>
#import <Adium/AIStatusControllerProtocol.h>
#import <Adium/AISharedAdium.h>
#import <Adium/AIAdiumProtocol.h>
#import <Adium/AIPreferenceControllerProtocol.h>
#import <AIUtilities/AIImageAdditions.h>

#import "AIStudiVZAccountViewController.h"
#import "AIStudiVZService.h"
#import "AIStudiVZAccount.h"
#import "ABStudiVZPlugin.h"


@implementation AIStudiVZService

//Account Creation
- (Class)accountClass{
	return [AIStudiVZAccount class];
}

//
- (AIAccountViewController *)accountViewController{
    return [AIStudiVZAccountViewController accountViewController];
}


//Service Description
- (NSString *)serviceCodeUniqueID{
	return SERVICE_CODE_UNIQUE_ID;
}
- (NSString *)serviceID{
	return SERVICE_ID;
}
- (NSString *)serviceClass{
	return SERVICE_CLASS;
}
- (NSString *)shortDescription{
	return SERVICE_SHORT_DESC;
}
- (NSString *)longDescription{
	return SERVICE_LONG_DESC;
}
/*
- (NSCharacterSet *)ignoredCharacters{
	return [NSCharacterSet characterSetWithCharactersInString:@" "];
}
*/

- (BOOL)caseSensitive{
	return NO;
}
- (BOOL)canRegisterNewAccounts{
	return NO;
}
- (AIServiceImportance)serviceImportance{
	return AIServiceSecondary;
}
/*!
 * @brief Placeholder string for the UID field
 */

- (NSString *)UIDPlaceholder
{
	return AILocalizedString(@"username@nimbuzz.com","Sample name and server for new Nimbuzz accounts");
}
- (NSString *)userNameLabel{
    return AILocalizedString(@"Nimbuzz ID",nil); //Jabber ID
}

- (NSImage *)defaultServiceIconOfType:(AIServiceIconType)iconType
{

	#if DEBUG_BUILD
//	AILogWithSignature(@"StudiVZService: getting service icon for type %@", [iconType description]);
	#endif
	
	NSImage *image;
	NSString *imPref = nil;
	NSSize imagesize;

	
	imPref = [[AIStudiVZAccount getSVZAccount] SVZAccountType];
	
	if (imPref == nil)
		imPref = [NSString stringWithString:@"s"];
	
	NSString *imageName = [imPref stringByAppendingString:@"vz"];

	if (iconType == AIServiceIconLarge){
	
		imagesize = NSMakeSize(48,48);

	}else if (iconType == AIServiceIconSmall) {
	
		imagesize = NSMakeSize(16,16);
		imageName = [imageName stringByAppendingString:@"-small"];

	} else {
		return nil;
	}
	
	image = [NSImage imageNamed:imageName forClass:[self class]];
	[image setSize:imagesize];
	
	
	return image;
}

- (BOOL)canCreateGroupChats
{
	return NO; 
}

- (BOOL)isSocialNetworkingService
{
	return YES;
}


- (BOOL)canSendOfflineMessageToContact:(AIListContact *)inContact
{
	return NO;
}

- (void)registerStatuses{
	[[adium statusController] registerStatus:STATUS_NAME_AVAILABLE
							 withDescription:[[adium statusController] localizedDescriptionForCoreStatusName:STATUS_NAME_AVAILABLE]
									  ofType:AIAvailableStatusType
								  forService:self];
	
}


@end
