//
//  CaptchaViewController.m
//  AdiumTestPlugin
//
//  Created by Vitus Lorenz-Meyer on 12.07.09.
//  Copyright 2009 Voodoo-Media. All rights reserved.
//

#import "CaptchaViewController.h"


@implementation CaptchaViewController

- (id)init
{
	self = [super initWithWindowNibName:@"CaptchaView"];
	return self;
}

- (id)initWithSVZid:(NSString *)svzid
{
	self = [self init];
	_svz_profile_id = [[svzid copy] retain];
	
	return self;
}


- (void)dealloc
{
	[_pic release];
	[_iv release];
	[_formKey release];
	[_svz_profile_id release];
	[super dealloc];
}

- (void)showWindowWithCaptcha:(NSImage *)pic formkey:(NSString *)theFormkey andIV: (NSString *)theIV
{

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZ:CaptchaViewCont(%@): Showing wind with pic: %@", _svz_profile_id, [pic description]);
	#endif

	[_pic release];
	_pic = [pic copy];

	[_iv release];
	_iv = [theIV copy];

	[_formKey release];
	_formKey = [theFormkey copy];
	
	[self showWindow:nil];
	
	NSWindow *win = [self window];
	[win makeKeyAndOrderFront:nil];
	[NSApp requestUserAttention:NSCriticalRequest];
}

- (void)windowDidLoad
{
    [super windowDidLoad];

	[imageview_captchaView setAllowsCutCopyPaste:false];
	[imageview_captchaView setEditable:false];
	[imageview_captchaView setImage:_pic];

}

- (BOOL)PostCaptcha:(NSString *)num{

	//build the post
	
	NSMutableDictionary* post_dict = [[NSMutableDictionary alloc] initWithCapacity:6];

	[post_dict setObject:@"Weiter" forKey:@"submit"];
	[post_dict setObject:num forKey:@"userCaptcha"];
	[post_dict setObject:_formKey forKey:@"formkey"];
	[post_dict setObject:_iv forKey:@"iv"];
	
	NSString* regData = [post_dict formatForHTTP];

	NSURL *url = [NSURL URLWithString: VZ_CAPTCHA_PAGE ];
	
	NSMutableURLRequest *post = [NSMutableURLRequest requestWithURL: url];
	
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZ:CaptchaViewCont(%@): posting captcha, post target is: %@", _svz_profile_id, [url description]);
//		NSString *len = [NSString stringWithFormat:@"%d", [regData length] ];
	//	AILogWithSignature( @"SVZProfileFetcher(%@): HTTBody for login is: %@, len: %@", SVZ_ProfileId, regData, len);
	#endif
	
	//now set the fields of the req
//	[post setValue: @"application/x-www-form-urlencoded" forHTTPHeaderField: @"Content-Type"];
//	[post setValue: len forHTTPHeaderField: @"Content-Length"];

	[post setValue: @"www.studivz.net" forHTTPHeaderField: @"Host"];
	[post setValue: VZ_CAPTCHA_PAGE forHTTPHeaderField: @"Referer"];
	[post setValue: FF_USER_AGENT_STR forHTTPHeaderField: @"User-Agent"];
	[post setHTTPMethod: @"POST"];
	[post setHTTPBody: [regData dataUsingEncoding: NSASCIIStringEncoding]];
	[post setCachePolicy: NSURLRequestReloadIgnoringLocalAndRemoteCacheData];			//set the cache policy to bypass local and remote caches, AFAP
	[post setTimeoutInterval: 180];														//set timeout to 3minutes

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZ:CaptchaViewCont(%@): Post captcha Req: %@ with body: %@", _svz_profile_id, post, regData);
	#endif

		
//	connection = [[NSURLConnection alloc] initWithRequest:post delegate:self];
	NSError *nse;
	NSHTTPURLResponse *nsur;
	
	NSData *data = [NSURLConnection sendSynchronousRequest:post returningResponse:&nsur error:&nse];

	NSString *receivedString = [[NSString alloc] initWithData:data encoding: NSUTF8StringEncoding];

	//check if the decoding failed
	if (receivedString == nil ||  [receivedString length] == 0){
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZ:CaptchaViewCont(%@): decoding string failed! returning...", _svz_profile_id);
		#endif

		return FALSE;
	}

	#if	DEBUG_BUILD
		#if 1
		//DBG_SHOW_FULL_REVC_HTML
			AILogWithSignature( @"SVZ:CaptchaViewCont(%@): post received: %@", _svz_profile_id, receivedString);
		#endif
		if (nsur != nil){
			AILogWithSignature(@"SVZ:CaptchaViewCont(%@): post ret resp. is %@", _svz_profile_id, [nsur description]);
		
			AILogWithSignature(@"SVZ:CaptchaViewCont(%@): code: %i", _svz_profile_id, [nsur statusCode]);
		}
//		if ([nsur allHeaderFields] != nil && [[nsur allHeaderFields] description] != nil)	
//			AILogWithSignature(@"SVZ:CaptchaViewCont(%@): header: %@", _svz_profile_id, [[nsur allHeaderFields] description]);
			
		if (nse != nil)
			AILogWithSignature( @"SVZ:CaptchaViewCont(%@): post ret error is %@", _svz_profile_id, [nse localizedDescription]);

	#endif

	
	if ([receivedString isMatchedByRegex: VZ_CAPTCHA_REGEX] && [receivedString isMatchedByRegex:VZ_CAPTCHA_FAILED]){
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZ:CaptchaViewCont(%@): captcha failed, aborting...", _svz_profile_id);
		#endif
		return FALSE;

	}
		
	return TRUE;

}



- (IBAction)OK:(id)sender
{
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZ:CaptchaViewCont(%@): OK pressed, textfield contains: %@", _svz_profile_id, [textfield_captchaInputField stringValue]);
	#endif

	[self closeWindow:nil];

	//do the post, yeah, do the POST!
	if ([self PostCaptcha:[textfield_captchaInputField stringValue]])
	
		//tell all observers that we solved the captcha
		[StudiVZProfileFetcher notifyAllObserversOfCaptchaSolve];
	
	else	//did not succeed, abort!
		[StudiVZProfileFetcher notifyAllObserversToDestroy];

}

- (IBAction)cancel:(id)sender
{
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZ:CaptchaViewCont(%@): cancel pressed, textfield contains: %@", _svz_profile_id, [textfield_captchaInputField stringValue]);
	#endif

	//do nothing, just tell all observers to destroy
	[StudiVZProfileFetcher notifyAllObserversToDestroy];
	
	//and tell ourselves to close the window
	[self closeWindow:nil];
	
}

/*!
 * @brief Close the window
 */
- (IBAction)closeWindow:(id)sender
{
	[super closeWindow:sender];
}


@end