//
//  StudiVZProfileFetcher.h
//  AdiumTestPlugin
//
//  Created by Vitus Lorenz-Meyer on 24.02.09.
//  Copyright 2009 derDoc. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <Foundation/NSURLConnection.h>
#import <Adium/AIAccount.h>
#import <RegexKit/RegexKit.h>

#import "AIStudiVZAccount.h"
#import "CaptchaViewController.h"

#define	SVZ_PROFILE_PAGE			@"http://www.studivz.net/Profile/"
#define	MVZ_PROFILE_PAGE			@"http://www.meinvz.net/Profile/"
#define	VZ_PROFILE_PAGE				@"http://www.%@vz.net/Profile/"

#define	SVZ_LOGIN_PAGE				@"https://secure.studivz.net/Login"
#define	MVZ_LOGIN_PAGE				@"https://secure.meinvz.net/Login"
#define	VZ_LOGIN_PAGE				@"https://secure.%@vz.net/Login"

#define VZ_DEF_PAGE					@"http://www.%@vz.net/Default"

#define VZ_NOT_LOGGED_IN_REGEX		@"(Bist Du schon drin?|Be part of it!)"
#define VZ_LOGGED_IN_REGEX			@"<title>(studi|mein)VZ \\| (Meine Startseite|My homepage)</title>"
#define VZ_CAPTCHA_TITLE			@"Gleich geht&#39;s weiter!"
#define VZ_CAPTCHA_REGEX			@"<title>(studi|mein)VZ \\| (Gleich geht&#39;s weiter!)</title>"
#define VZ_CAPTCHA_PAGE				@"http://www.studivz.net/Captcha"
#define VZ_CAPTCHA_FAILED			@"Die Eingabe war nicht korrekt. Bitte versuch' es doch nochmal"

#define MAX_LOGINS_ATTEMPTS			3
#define MAX_FETCH_ATTEMPTS			3
#define SET_NAME_FROM_PROFILE		1													//this causes the alias to be set from the fetched profile, but only if there is none set already

#define PROFILE_MVZ_REGEX			@"Img/Logos/mvzLogo15px.gif"
#define PROFILE_SVZ_REGEX			@"Img/Logos/svzLogo15px.gif"
#define EXPL_SET_BUD_SRV_PIC_IF_DIF	0													//explicitly set the default SVZ icon as server-side when it is different from account type icon?
#define	DBG_SHOW_FULL_REVC_HTML		0													//show the full received HTML in debug output?

//Regexes
#define LOGIN_FORMKEY_REGEX			@"name=\"formkey\"\\svalue=\"([0-9a-f]+)\""
#define LOGIN_IV_REGEX				@"name=\"iv\"\\svalue=\"([0-9a-f]+)\""
#define PROFILE_ALIAS_REGEX			@"<title>(studi|mein)VZ \\| (.+)</title>"
#define	PROFILE_PIC_REGEX			@"<img src=\"(.+)\" class=\"obj-profileImage\" id=\"profileImage\""
#define	PROFILE_STATUS_REGEX		@"<div id=\"microblogContent\">(\\v|\\s)*<p class=\"microblogHistory\">(.*?)</p>"
#define	CAPTCHA_REGEX				@"src=\"(.*?)\" id=\"captchaImage\""
#define VZ_NOPIC					@"vz-nopic.jpg"

#define FF_USER_AGENT_STR			@"Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1) Gecko/20090624"



	static BOOL				performingLogin;
	static BOOL				UserNotifiedAboutCaptcha;
	static NSMutableArray	*loginObservers;
	static NSMutableArray	*captchaObservers;
	static BOOL				performingCaptcha;


@interface StudiVZProfileFetcher : NSObject {
	NSString		 *SVZ_user;
	NSString		 *SVZ_pass;
	AIStudiVZAccount *SVZ_Account;


	NSString		 *SVZ_ProfileId;
	NSString		 *profile_url;
	AIListContact	 *SVZ_Contact;
	
	int				 numLoginAttempts;
	int				 numFetchAttempts;
	BOOL			 selfExecutingLogin;
	BOOL			 waitingForLogin;

	BOOL			 selfSolvingCaptcha;
	BOOL			 waitingForCaptcha;
	
	NSMutableData	 *receivedData;
	NSURLConnection  *connection;
}
//statics
+ (void) initialize;
+ (void) notifyAllObserversOfLogin;
+ (void) notifyAllObserversToDestroy;
+ (void) notifyAllObserversOfCaptchaSolve;

//init and stuff
- (id) init;
- (void) destroy;
- (void) dealloc;
- (void) SetSVZAccount:(AIStudiVZAccount *)theAccount;
- (StudiVZProfileFetcher*) initWithContact:(AIListContact *)theContact;
- (StudiVZProfileFetcher*) initWithContact:(AIListContact *)theContact andAccount:(AIStudiVZAccount *)theAccount;

//functions to check that the necc. instance variables are still there
- (void) PreEventloopVarScopeCheck;
- (void) PostEventloopVarRefcountCheck;
- (id)   setVarsFromContact: (AIListContact *)theContact;

//functions that do the actual work
- (void)fetchProfile;
- (void)executeLoginWithFormKey: (NSString *)theFormkey andIV: (NSString *)theIV;
- (NSImage *) fetchPicWithURL: (NSString *)theurl;

@end
