//
//  StudiVZProfileFetcher.m
//  AdiumTestPlugin
//
//  Created by Vitus Lorenz-Meyer on 24.02.09.
//  Copyright 2009 derDoc. All rights reserved.
//

#import <AIUtilities/AIImageAdditions.h>

#import "StudiVZProfileFetcher.h"
#import "curl_extra_dict.h"
#import "AIStudiVZAccountViewController.h"


@implementation StudiVZProfileFetcher		//(NSURLConnectionDelegate)

+ (void) initialize {

	performingLogin = false;
	loginObservers = [[NSMutableArray arrayWithCapacity:3] retain];

}

- (void) SetSVZAccount:(AIStudiVZAccount *)theAccount {

	if (SVZ_Account == nil){
		SVZ_Account = theAccount;
		SVZ_user =	[theAccount preferenceForKey:KEY_SVZ_ACCOUNT_NAME group:GROUP_ACCOUNT_STATUS];
		SVZ_pass =	[theAccount preferenceForKey:KEY_SVZ_ACCOUNT_PASS group:GROUP_ACCOUNT_STATUS];
	}
}


+ (void) notifyAllObserversOfLogin {

	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher(): notifying all waiting observers...");
	#endif
	
	if (loginObservers && [loginObservers isKindOfClass:[NSMutableArray class]]){
		for (StudiVZProfileFetcher *observer in loginObservers) {
			if (observer != nil && [observer isKindOfClass:[self class]]){
				@try{
					[observer fetchProfile];
				} @catch ( NSException *e ) {
					AILog( @"SVZProfileFetcher(%@): Exception during notifying of observers %@: %@\n", [e name], [e reason] );
				}
			}
		}
	}
	
	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher(): done notifying...");
	#endif

}

- (id) init {
    self = [super init];
    return self;
}


- (void) dealloc
{
	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher: dealloc called on self");
	#endif

	@try{
//		[super dealloc];
	} @catch ( NSException *e ) {
		AILog( @"SVZProfileFetcher(%@): Exception deallocing super %@: %@\n", [e name], [e reason] );
	}
}

- (StudiVZProfileFetcher*) initWithContact:(AIListContact *)theContact		//andAccount:(AIStudiVZAccount *)theAccount
{
	self = [super init];

	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher: initializing for: %@", theContact);
	#endif
	
	waitingForLogin = false;
	selfExecutingLogin = false;
	numLoginAttempts = 0;
	SVZ_Contact = theContact;


	if ( [[theContact UID] hasSuffix:@"studivz.nimbuzz.com"] ){
		SVZ_ProfileId = [[theContact UID] substringToIndex:16];

	#if DEBUG_BUILD
//		AILog( @"SVZProfileFetcher(%@) init: fetching profile for id: %@", SVZ_ProfileId, SVZ_ProfileId);
	#endif
		profile_url = [SVZ_PROFILE_PAGE stringByAppendingString:SVZ_ProfileId];
		[SVZ_Contact setFormattedUID:SVZ_ProfileId notify:NotifyNow];
	}
	return self;
}


- (StudiVZProfileFetcher*) initWithContact:(AIListContact *)theContact andAccount:(AIStudiVZAccount *)theAccount
{
	self = [self initWithContact:theContact];
	
	SVZ_Account = theAccount;
	SVZ_user =	[theAccount preferenceForKey:KEY_SVZ_ACCOUNT_NAME group:GROUP_ACCOUNT_STATUS];
	SVZ_pass =	[theAccount preferenceForKey:KEY_SVZ_ACCOUNT_PASS group:GROUP_ACCOUNT_STATUS];
	
	return self;
}


//Executes a login to the SVZ website if we are not already logged in
- (void)executeLoginWithFormKey: (NSString *)theFormkey andIV: (NSString *)theIV
{
	//return if we are already waiting for a login
	if (performingLogin && !selfExecutingLogin){
		#if DEBUG_BUILD
		AILog( @"SVZProfileFetcher(%@): already waiting for login...", SVZ_ProfileId);
		#endif
		//and add ourselves to the list of objects waiting to be notified of a successfull login
		if (!waitingForLogin && ![loginObservers containsObject:self]){
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): adding self to Observer List...", SVZ_ProfileId);
			#endif
			[loginObservers addObject:self];
			waitingForLogin = true;
		}
		return;
	}

	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher(%@): logging in with formkey: %@ and iv: %@", SVZ_ProfileId, theFormkey, theIV);
	#endif

	performingLogin = true;
	selfExecutingLogin = true;
	
	NSMutableDictionary* post_dict = [[NSMutableDictionary alloc] initWithCapacity:6];
	[post_dict setObject:SVZ_user forKey:@"email"];
	[post_dict setObject:SVZ_pass forKey:@"password"];
	[post_dict setObject:@"Einloggen" forKey:@"login"];
	[post_dict setObject:@"false" forKey:@"jsEnabled"];
	[post_dict setObject:theFormkey forKey:@"formkey"];
	[post_dict setObject:theIV forKey:@"iv"];
	
	NSString* regData = [post_dict formatForHTTP];


	NSMutableURLRequest *post = [NSMutableURLRequest requestWithURL: [NSURL URLWithString: SVZ_LOGIN_PAGE]];
	
	#if DEBUG_BUILD
	NSString *len = [NSString stringWithFormat:@"%d", [regData length] ];
//	AILog( @"SVZProfileFetcher(%@): HTTBody for login is: %@, len: %@", SVZ_ProfileId, regData, len);
	#endif

	[post setValue: @"application/x-www-form-urlencoded" forHTTPHeaderField: @"Content-Type"];
//	[post setValue: len forHTTPHeaderField: @"Content-Length"];
	[post setValue: @"http://www.studivz.net/Default" forHTTPHeaderField: @"Referer"];
	[post setHTTPMethod: @"POST"];
	[post setHTTPBody: [regData dataUsingEncoding: NSASCIIStringEncoding]];

	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher(%@): Post Req. for login is: %@", SVZ_ProfileId, post);
	#endif


	if (connection != nil)
		[connection release];
	connection = [[NSURLConnection alloc] initWithRequest:post delegate:self];


	//and create the receiving buffer
	if (connection) {
		#if DEBUG_BUILD
		AILog( @"SVZProfileFetcher(%@): Connection is: %@, creating receive buffer", SVZ_ProfileId, connection);
		#endif
		if (receivedData != nil)
			[receivedData release];
		receivedData = [[NSMutableData data] retain];
	}

	[post_dict release];

}

//attempts to fetch a profile for the id that this class intance represents
- (void)fetchProfile
{

	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher(%@): fetching profile for: %@",SVZ_ProfileId, self);
	#endif

	@try{
		NSURLRequest *theRequest=[NSURLRequest requestWithURL:[NSURL URLWithString: profile_url]
                        cachePolicy:NSURLRequestUseProtocolCachePolicy
						timeoutInterval:60.0];

		if (connection != nil)
			[connection release];
		connection = [ [NSURLConnection alloc] initWithRequest:theRequest delegate:self];

		if (receivedData != nil)
			[receivedData release];
		receivedData = [ [NSMutableData data] retain];

	} @catch ( NSException *e ) {
			AILog( @"SVZProfileFetcher(%@): Exception setting status %@: %@\n", [e name], [e reason] );
	}

}


#pragma mark delegates

- (void)connection:(NSURLConnection *)theConnection didReceiveResponse:(NSURLResponse *)response{ 

	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher(%@): connection %@ successful with response: %@", SVZ_ProfileId, theConnection, response);
	#endif
	@try{
//		AILog( @"SVZProfileFetcher(%@): setting receivedData to 0: %@", SVZ_ProfileId, [receivedData description]);
		[receivedData setLength:0];
	} @catch ( NSException *e ) {
			AILog( @"SVZProfileFetcher(%@): Exception setting receivedData to 0 %@: %@\n", [e name], [e reason] );
	}
//		AILog( @"SVZProfileFetcher(%@): receivedData set to: %@, and returning", SVZ_ProfileId, [receivedData description]);

}

- (void)connection:(NSURLConnection *)theConnection didReceiveData:(NSData *)data{
	
	#if DEBUG_BUILD
//	AILog( @"SVZProfileFetcher(%@): connection receivedData: %@, appending", SVZ_ProfileId, [data description]);
	#endif

	@try{
		[receivedData appendData:data];
	} @catch ( NSException *e ) {
			AILog( @"SVZProfileFetcher(%@): Exception appending receivedData to buffer %@: %@\n", [e name], [e reason] );
	}

	#if DEBUG_BUILD
//	AILog( @"SVZProfileFetcher(%@): connection receivedData returning", SVZ_ProfileId);
	#endif

}

- (void)connection:(NSURLConnection *)theConnection didFailWithError:(NSError *)error
{

	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher(%@): connection %@ failed with error %@", SVZ_ProfileId, theConnection, error);
	#endif

}

/* our connection will get forwarded to another adress as per 301 received from server, this gives us a chance to allow or abort */
-(NSURLRequest *)connection:(NSURLConnection *)theConnection
            willSendRequest:(NSURLRequest *)request
           redirectResponse:(NSURLResponse *)redirectResponse

{
	NSURLRequest * newrequest = request;
	
    if (redirectResponse) {
	
		#if DEBUG_BUILD
		AILog( @"SVZProfileFetcher(%@): connection %@ got response %@ to redirect to %@", SVZ_ProfileId, theConnection, redirectResponse, request);
		#endif
		
		//if we get forwarded to an URL ending w/ /Start, then we were sucessfully logged in, since this never returns anything, dissallow and re-request profile
		if ([[[request URL] absoluteString] hasSuffix:@"Start"] ){
			newrequest = nil;
			
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): disallowing redirect", SVZ_ProfileId);
			AILog( @"SVZProfileFetcher(%@): login successfull!", SVZ_ProfileId);
			#endif
			
			[self fetchProfile];
			//and notify all waiting objects that we are logged in now 
			[StudiVZProfileFetcher notifyAllObserversOfLogin];
			selfExecutingLogin = false;
			performingLogin = false;

		}
    }

    return newrequest;
}



- (void)connectionDidFinishLoading:(NSURLConnection *)theConnection{ 
	    
	#if DEBUG_BUILD
	AILog( @"SVZProfileFetcher(%@): Download succeeded! Received %d bytes of data",SVZ_ProfileId, [receivedData length]);
	#endif

	NSString *receivedString = NULL;
	BOOL	  bDone = false;
	
	receivedString = [[NSString alloc] initWithData:receivedData encoding: NSUTF8StringEncoding];
		
	#if	DEBUG_BUILD
		#if DBG_SHOW_FULL_REVC_HTML
			AILog( @"SVZProfileFetcher(%@): received: %@", SVZ_ProfileId, receivedString);
		#endif
	#endif

	//check if we were logging in or making a request for the profile
	if (selfExecutingLogin){
	
		//and now check if we were successfully logged in by checking for the homepage mark
		if (([receivedString rangeOfString:SVZ_LOGGED_IN_MARK].location != NSNotFound)){
			//we were logged in, then re-request the profile
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): login successfull!", SVZ_ProfileId);
			#endif
			[self fetchProfile];

		}else{
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): login attempt#%d unsuccessfull :( ", SVZ_ProfileId, numLoginAttempts);
			#endif
			numLoginAttempts++;
			
			if (numLoginAttempts < MAX_LOGINS_ATTEMPTS){
				#if	DEBUG_BUILD
				AILog( @"SVZProfileFetcher(%@): proceeding to attempt #%d", SVZ_ProfileId, numLoginAttempts);
				#endif
				[self fetchProfile];
			}
		}
		
	}else{
		//we were fetching the profile page, lets take a look at the result then, shall we?
		//check if we need to log in
		if (([receivedString rangeOfString:SVZ_NOT_LOGGED_IN_MARK].location != NSNotFound)){
			//do the regex magic
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): not logged in!", SVZ_ProfileId);
			#endif
			
			NSString *formKey = NULL;
			NSString *iv = NULL;
			
			[receivedString getCapturesWithRegexAndReferences: LOGIN_FORMKEY_REGEX, 
											@"$1", &formKey, 
											nil]; 

			[receivedString getCapturesWithRegexAndReferences: LOGIN_IV_REGEX, 
											@"$1", &iv, 
											nil]; 
			
			//we are not logged in, login now
			[self executeLoginWithFormKey: formKey andIV: iv];
			
		}else{
		
			//we do not need to log in, we got the profile, yeha!
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): profile fetch successfull!", SVZ_ProfileId);
			#endif			
			
			NSString *Alias = nil;
			NSString *PicURL = nil;
			NSString *RemoteGroup = [NSString stringWithString:@"StudiVZ"];
			NSString *net = [NSString stringWithString:@"svz"];
			NSString *RemoteStatus = nil;
			

			[receivedString getCapturesWithRegexAndReferences: PROFILE_ALIAS_REGEX, 
											@"$1", &Alias, 
											nil]; 

			[receivedString getCapturesWithRegexAndReferences: PROFILE_PIC_REGEX, 
											@"$1", &PicURL, 
											nil];
			
			[receivedString getCapturesWithRegexAndReferences: PROFILE_STATUS_REGEX,
											@"$2", &RemoteStatus,
											nil];


			if ([receivedString isMatchedByRegex: PROFILE_MVZ_REGEX]){
				RemoteGroup = [NSString stringWithString:@"MeinVZ"];
				net = [NSString stringWithString:@"mvz"];
			}
			
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): got Alias: %@", SVZ_ProfileId, Alias);
			#endif

			//set the display name now
			#if SET_NAME_FROM_PROFILE

				if ([SVZ_Contact displayName] == nil || [SVZ_Contact displayName] == [SVZ_Contact formattedUID]){
					#if DEBUG_BUILD
					AILog( @"SVZProfileFetcher(%@): setting Alias: %@", SVZ_ProfileId, Alias);
					#endif
					
					[SVZ_Contact setDisplayName:Alias];
				}
				
			#endif
			
			
			//set the group
			@try {
				#if DEBUG_BUILD
				AILog( @"SVZProfileFetcher(%@): got Group: %@", SVZ_ProfileId, RemoteGroup);
				#endif
				
				[SVZ_Contact setRemoteGroupName:RemoteGroup];
				#if DEBUG_BUILD
				AILog( @"SVZProfileFetcher(%@): got Status: %@", SVZ_ProfileId, RemoteStatus);
				#endif
				
				//and the status
				if (RemoteStatus != nil){
					#if DEBUG_BUILD
					AILog( @"SVZProfileFetcher(%@): setting Status to: %@", SVZ_ProfileId, RemoteStatus);
					#endif
					
					[SVZ_Contact setStatusMessage:[[[NSAttributedString alloc] initWithString:RemoteStatus] autorelease] notify:NotifyLater];
				}
			} @catch ( NSException *e ) {
				AILog( @"SVZProfileFetcher(%@): Exception setting status %@: %@\n", [e name], [e reason] );
			}

			//and lastly download the pic
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): got Pic URL: %@", SVZ_ProfileId, PicURL);
			#endif			
			
			
			NSImage *image = nil;
			
			if ([PicURL hasSuffix:@"svz-nopic.jpg"]){
				//do nothing in this case, coz it will fall through to the service icon if needed
			}else{
				//try getting the image
				@try{
					#if DEBUG_BUILD
					AILog( @"SVZProfileFetcher(%@): requesting pic from URL: %@", SVZ_ProfileId, PicURL);
					#endif			

					image = [[NSImage alloc] initWithContentsOfURL: [NSURL URLWithString: PicURL] ];
				} @catch ( NSException *e ) {
					AILog( @"SVZProfileFetcher(%@): Exception fetching profile pic %@: %@\n", [e name] , [e reason]  );
				}
			}
			
			#if EXPL_SET_BUD_SRV_PIC_IF_DIF
			if (image == nil || ![image isValid]){
				NSString *imPref = [SVZ_Account SVZAccountType];
				
				NSString *imageName = [imPref stringByAppendingString:@"vz"];

				if (![net isEqualToString: imageName])
					image =	[NSImage imageNamed:imageName forClass:[self class]];
				
				#if DEBUG_BUILD
				AILog( @"SVZProfileFetcher(%@): falling back to service Icon...", SVZ_ProfileId);
				#endif
				
			}
			#endif
			
			if (image != nil && [image isValid]){
				@try {
					
					#if DEBUG_BUILD
					AILog( @"SVZProfileFetcher(%@): setting account pic...", SVZ_ProfileId);
					#endif
					
					//set it
					NSData *userIconData = [image TIFFRepresentation];
					if ([userIconData length] > 10){
						[SVZ_Contact setServersideIconData:userIconData notify:NotifyLater];
					}
							
				} @catch ( NSException *e ) {
					AILog( @"SVZProfileFetcher(%@): Exception setting profile pic %@: %@\n", [e name], [e reason] );
				}
			}

			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): notifying buddy of update...", SVZ_ProfileId);
			#endif
			
			[SVZ_Contact notifyOfChangedPropertiesSilently:YES];
			
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): cleaning up...", SVZ_ProfileId);
			#endif
			
			
			//and remove ourselves from the notify object if we were in it, this results in a release msg send to ourselves
			@try {
				if (waitingForLogin && (loginObservers != nil) && ([loginObservers count] > 0) && ([loginObservers containsObject:self]) ){
					#if DEBUG_BUILD
					AILog( @"SVZProfileFetcher(%@): removing self from loginObservers...", SVZ_ProfileId);
					#endif
					waitingForLogin = false;
					[loginObservers removeObject:self];
					
				}else{
					#if DEBUG_BUILD
					AILog( @"SVZProfileFetcher(%@): releasing self...", SVZ_ProfileId);
					#endif
					
				}	
			} @catch ( NSException *e ) {
				AILog( @"SVZProfileFetcher(%@): Exception releasing... %@: %@\n", [e name], [e reason] );
			}
	
			// and lastly release the connection, and the data object, and ourselves
			#if DEBUG_BUILD
			AILog( @"SVZProfileFetcher(%@): releasing connection and receivedData", SVZ_ProfileId);
			#endif

			[connection release];
			[receivedData release];
			[self autorelease];

		}
	}
}


@end
