//
//  StudiVZProfileFetcher.m
//  AdiumTestPlugin
//
//  Created by Vitus Lorenz-Meyer on 24.02.09.
//  Copyright 2009 derDoc. All rights reserved.
//

#import <AIUtilities/AIImageAdditions.h>

#import "StudiVZProfileFetcher.h"
#import "curl_extra_dict.h"
#import "AIStudiVZAccountViewController.h"


@implementation StudiVZProfileFetcher		//(NSURLConnectionDelegate)

+ (void) initialize {

	performingLogin = false;
	performingCaptcha = false;
	UserNotifiedAboutCaptcha = false;
	loginObservers = [[NSMutableArray arrayWithCapacity:3] retain];
	captchaObservers = [[NSMutableArray arrayWithCapacity:3] retain];

}

- (void) SetSVZAccount:(AIStudiVZAccount *)theAccount {

	if (SVZ_Account == nil){
		SVZ_Account = theAccount;
		SVZ_user =	[theAccount preferenceForKey:KEY_SVZ_ACCOUNT_NAME group:GROUP_ACCOUNT_STATUS];
		SVZ_pass =	[theAccount preferenceForKey:KEY_SVZ_ACCOUNT_PASS group:GROUP_ACCOUNT_STATUS];
	}
}

+ (void) notifyAllObserversOfCaptchaSolve {

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(): notifying all waiting observers of sucessful captcha solve.");
	#endif
	
	if (captchaObservers && [captchaObservers isKindOfClass:[NSMutableArray class]]){
		for (StudiVZProfileFetcher *observer in captchaObservers) {
			if (observer != nil && [observer isKindOfClass:[self class]]){
				@try{
					[observer fetchProfile];
					#if DEBUG_BUILD
						AILogWithSignature( @"SVZProfileFetcher(): notifying: %@", [observer description]);
					#endif

				} @catch ( NSException *e ) {
					AILogWithSignature( @"SVZProfileFetcher(notify): Exception during notifying of observers %@: %@\n", [e name], [e reason] );
				}
			}
		}
	}
	
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(): done notifying...");
	#endif

}


+ (void) notifyAllObserversOfLogin {

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(): notifying all waiting observers of sucessful login.");
	#endif
	
	if (loginObservers && [loginObservers isKindOfClass:[NSMutableArray class]]){
		for (StudiVZProfileFetcher *observer in loginObservers) {
			if (observer != nil && [observer isKindOfClass:[self class]]){
				@try{
					[observer fetchProfile];
					#if DEBUG_BUILD
						AILogWithSignature( @"SVZProfileFetcher(): notifying: %@", [observer description]);
					#endif

				} @catch ( NSException *e ) {
					AILogWithSignature( @"SVZProfileFetcher(notify): Exception during notifying of observers %@: %@\n", [e name], [e reason] );
				}
			}
		}
	}
	
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(): done notifying...");
	#endif

}

+ (void) notifyAllObserversToDestroy {

	int i;
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(): notifying all waiting observers to cease operation and destroy.");
	#endif
	
	@try{
		if (loginObservers && [loginObservers isKindOfClass:[NSMutableArray class]] && [loginObservers count] > 0 ){
			for(i = ([loginObservers count]-1); i >= 0 ; i--) {
			//for (StudiVZProfileFetcher *observer in loginObservers) {
                id observer = [loginObservers objectAtIndex:i];
			
				if (observer != nil && [observer isKindOfClass:[self class]]){
					@try{
						#if DEBUG_BUILD
							AILogWithSignature( @"SVZProfileFetcher(): notifying following fetcher: %@", [observer description]);
						#endif	
						//remove the current obj
						[loginObservers removeObjectAtIndex:i];

						//and also remove from captcha obs if in there
						if ([captchaObservers containsObject:observer])
							[captchaObservers removeObject: observer];

						//and destroy
						[observer destroy];
						

					} @catch ( NSException *e ) {
						AILogWithSignature( @"SVZProfileFetcher(destroy-notify): Exception during destroy-notifying of observers %@: %@\n", [e name], [e reason] );
					} 
					
				}
			}
		}
	} @catch ( NSException *e ) {
		AILogWithSignature( @"SVZProfileFetcher(destroy-notify): Exception during enumeration of observers %@: %@\n", [e name], [e reason] );
	}
	
	//now destroy left captcha observers
	if (captchaObservers && [captchaObservers isKindOfClass:[NSMutableArray class]]  && [captchaObservers count] > 0 ){
		for(i = ([captchaObservers count]-1); i >= 0 ; i--) {
			id observer = [captchaObservers objectAtIndex:i];
			
			if (observer != nil && [observer isKindOfClass:[self class]]){
				@try{
					#if DEBUG_BUILD
						AILogWithSignature( @"SVZProfileFetcher(): notifying following fetcher: %@", [observer description]);
					#endif	
					//remove the current obj
					[captchaObservers removeObjectAtIndex:i];
					//and destroy
					[observer destroy];

				} @catch ( NSException *e ) {
					AILogWithSignature( @"SVZProfileFetcher(notify): Exception during notifying of observers %@: %@\n", [e name], [e reason] );
				}
			}
		}
	}

	
	//this instance made the call, and all other objects have already been destroyed, now clear the captcha flag
	UserNotifiedAboutCaptcha = false;
	
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(): done notify destroying...");
	#endif

}


- (id) init {
    self = [super init];
    return self;
}


- (void)destroy
{
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): destroying self, cleaning up...", SVZ_ProfileId);
	#endif
			
			
	//and remove ourselves from the notify object if we were in it, this results in a release msg send to ourselves
	@try {
		if (waitingForLogin && (loginObservers != nil) && ([loginObservers count] > 0) && ([loginObservers containsObject:self]) ){
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): removing self from loginObservers...", SVZ_ProfileId);
			#endif
			
			waitingForLogin = false;
			[loginObservers removeObject:self];
					
		}else{
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): not in login-observers... doing nothing", SVZ_ProfileId);
			#endif
					
		}	
	} @catch ( NSException *e ) {
		AILogWithSignature( @"SVZProfileFetcher(%@): Exception removing from login-observers... %@: %@\n", SVZ_ProfileId, [e name], [e reason] );
	}

	//and remove ourselves from the notify object if we were in it, this results in a release msg send to ourselves
	@try {
		if (waitingForCaptcha && (captchaObservers != nil) && ([captchaObservers count] > 0) && ([captchaObservers containsObject:self]) ){
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): removing self from loginObservers...", SVZ_ProfileId);
			#endif
			
			waitingForCaptcha = false;
			[captchaObservers removeObject:self];
					
		}else{
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): not in login-observers... doing nothing", SVZ_ProfileId);
			#endif
					
		}	
	} @catch ( NSException *e ) {
		AILogWithSignature( @"SVZProfileFetcher(%@): Exception removing from login-observers... %@: %@\n", SVZ_ProfileId, [e name], [e reason] );
	}


	// and lastly release the connection, and the data object, and ourselves
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(): releasing connection and receivedData and returning...");
	#endif
	
	@try {

		[connection release]; connection = nil;
		[receivedData release]; receivedData = nil;
		
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): releasing self...", SVZ_ProfileId);
		#endif
		[self release];
		
	} @catch ( NSException *e ) {
		AILogWithSignature( @"SVZProfileFetcher(%@): Exception releasing... %@: %@\n", SVZ_ProfileId, [e name], [e reason] );
	}

}



- (void) dealloc
{
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher: dealloc called on self");
	#endif

	@try{
		[super dealloc];
	} @catch ( NSException *e ) {
		AILogWithSignature( @"SVZProfileFetcher(%@): Exception deallocing super %@: %@\n", SVZ_ProfileId, [e name], [e reason] );
	}
}

- (StudiVZProfileFetcher*) initWithContact:(AIListContact *)theContact	
{
	self = [super init];

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(init): initializing for contact: %@", theContact);
	#endif
	
	waitingForLogin = false;
	waitingForCaptcha = false;
	selfExecutingLogin = false;
	selfSolvingCaptcha = false;
	numLoginAttempts = 0;
	numFetchAttempts = 0;
	SVZ_Contact = theContact;
	SVZ_ProfileId = @"";
	profile_url = @"";
	
	if ( [[theContact UID] hasSuffix:@"studivz.nimbuzz.com"] )
		self = [self setVarsFromContact:theContact];
	else
		return nil;
		
	[SVZ_Contact setFormattedUID:SVZ_ProfileId notify:NotifyNow];

	return self;

}

	//this does a quick check if all the vars are still valid and accessible
- (void) PreEventloopVarScopeCheck{

	//and just try to reset the names and URLs, coz that seems to get confused on a rerun sometimes, but only if it does not look right
	#if DEBUG_BUILD
//		AILogWithSignature( @"SVZProfileFetcher(PreEventLoop): checking Profile_ID and profile_url for prefix http and suffix pid");
	#endif
	
	@try{
		
		[profile_url isKindOfClass:[NSString class]];
		[SVZ_ProfileId isKindOfClass:[NSString class]];
		
	} @catch ( NSException *e ) {
			AILogWithSignature( @"SVZProfileFetcher(PreEventLoop): Exception trying to access prof_url %@: %@. Resetting UIDs from contact.\n", [e name], [e reason] );
			[self setVarsFromContact:SVZ_Contact];
	}

	//now check that they are valid
	bool valid = true;
	
	if (	(SVZ_ProfileId == nil) || 
			!([SVZ_ProfileId respondsToSelector: @selector(isKindOfClass)]) ||
			!([SVZ_ProfileId isKindOfClass:[NSString class]]) ||
			!([SVZ_ProfileId length] == 16)
		){
			valid = false;
			#if DEBUG_BUILD
//				AILogWithSignature( @"SVZProfileFetcher(PreRunLoop): SVZ_ProfileId is invalid! resetting!");
			#endif

		}
	
	if (	(profile_url == nil) || 
			!([profile_url respondsToSelector: @selector(isKindOfClass)]) ||
			!([profile_url isKindOfClass:[NSString class]]) ||
			!([profile_url hasPrefix:@"http://"]) || 
			!([profile_url hasSuffix:SVZ_ProfileId])
		){
			#if DEBUG_BUILD
//				AILogWithSignature( @"SVZProfileFetcher(PreRunLoop): profile_url is invalid! resetting!");
			#endif
		
			valid = false;
		}
		
	if (!valid)
		[self setVarsFromContact:SVZ_Contact];


}

//this just checks that prof_url and prof_id do not get disposed of at the end of the event loop
- (void) PostEventloopVarRefcountCheck{

	if ([profile_url retainCount] == 1){
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(PostRunLoop): profile_url retain-count is 1! Adding 1.");
		#endif
		//inc by one
		[profile_url retain];
		
	}

	if ([SVZ_ProfileId retainCount] == 1){
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(PostRunLoop): SVZ_ProfileId retain-count is 1! Adding 1.");
		#endif
		//inc by one
		[SVZ_ProfileId retain];

	}
}




- (id) setVarsFromContact:(AIListContact *)theContact
{

		SVZ_ProfileId = [[[[theContact UID] copyWithZone:nil] substringToIndex:16] retain];
		
		NSString *prof_page = [NSString stringWithString:SVZ_PROFILE_PAGE];
		if (SVZ_Account != nil)
			prof_page = [NSString stringWithFormat: VZ_PROFILE_PAGE, [SVZ_Account SVZLongAccountType] ];
			
		profile_url = [[prof_page stringByAppendingString:SVZ_ProfileId] retain];
		
		#if DEBUG_BUILD
//			AILogWithSignature( @"SVZProfileFetcher(set Names): getting names from contact, profile_id is: %@, prof_url is %@", SVZ_ProfileId, profile_url);
		#endif
		return self;
}


- (StudiVZProfileFetcher*) initWithContact:(AIListContact *)theContact andAccount:(AIStudiVZAccount *)theAccount
{
	SVZ_Account = theAccount;
	self = [self initWithContact:theContact];
	
	SVZ_user =	[theAccount preferenceForKey:KEY_SVZ_ACCOUNT_NAME group:GROUP_ACCOUNT_STATUS];
	SVZ_pass =	[theAccount preferenceForKey:KEY_SVZ_ACCOUNT_PASS group:GROUP_ACCOUNT_STATUS];
	
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(init): initializing w/ account: %@m, user/pass: '%@':'%@'", theAccount, SVZ_user, SVZ_pass);
		AILogWithSignature( @"SVZProfileFetcher(init): prof_id set to: '%@'", SVZ_ProfileId);
	#endif

	return self;
}

//get and solve a captcha with the help of the Captcha View
- (void)solveCaptchaFromPage: (NSString *)thePage
{

	//return if we are already waiting for a login
	if (performingCaptcha && !selfSolvingCaptcha){
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): already waiting for captcha...", SVZ_ProfileId);
		#endif
		//and add ourselves to the list of objects waiting to be notified of a successfull login
		if (!waitingForCaptcha && ![captchaObservers containsObject:self]){
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): adding self to Observer List...", SVZ_ProfileId);
			#endif
			[captchaObservers addObject:self];
			waitingForCaptcha = true;
		}
		return;
	}

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): getting captcha from page", SVZ_ProfileId);
	#endif

	performingCaptcha = true;
	selfSolvingCaptcha = true;

	//add ourselves to the listeners
	[captchaObservers addObject:self];

	//now parse the captcha out, get the pic and init the Captcha View
	NSString *captcha_url = NULL;	//[NSString stringWithString:@"test-init"];
	NSString *formKey = NULL;
	NSString *iv = NULL;
			
	[thePage getCapturesWithRegexAndReferences: LOGIN_FORMKEY_REGEX, 
				@"$1", &formKey, 
				nil]; 

	[thePage getCapturesWithRegexAndReferences: LOGIN_IV_REGEX, 
				@"$1", &iv, 
				nil]; 
	
	[thePage getCapturesWithRegexAndReferences: CAPTCHA_REGEX, 
				@"$1", &captcha_url, nil]; 

	#if DEBUG_BUILD
		if (captcha_url != nil){
			AILogWithSignature(@"SVZProfileFetcher(%@): getting captcha pic from URL: %@, with fk: %@ and iv: %@", SVZ_ProfileId, captcha_url, formKey, iv);
		}else{
			AILogWithSignature( @"SVZProfileFetcher(%@): getting captcha pic from URL: nil", SVZ_ProfileId);
		}
	#endif

	//get the pic
	NSImage *cap = [[NSImage alloc] initWithContentsOfURL:[NSURL URLWithString:[@"http://www.studivz.net" stringByAppendingString:captcha_url ]]];
	
	//and show the controller
	CaptchaViewController *cvc = [[CaptchaViewController alloc] initWithSVZid:SVZ_ProfileId];
	[cvc showWindowWithCaptcha:cap formkey:formKey andIV:iv];
	
	
}


//Executes a login to the SVZ website if we are not already logged in
- (void)executeLoginWithFormKey: (NSString *)theFormkey andIV: (NSString *)theIV
{
	//return if we are already waiting for a login
	if (performingLogin && !selfExecutingLogin){
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): already waiting for login...", SVZ_ProfileId);
		#endif
		//and add ourselves to the list of objects waiting to be notified of a successfull login
		if (!waitingForLogin && ![loginObservers containsObject:self]){
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): adding self to Observer List...", SVZ_ProfileId);
			#endif
			[loginObservers addObject:self];
			waitingForLogin = true;
		}
		return;
	}

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): logging in with formkey: %@ and iv: %@", SVZ_ProfileId, theFormkey, theIV);
	#endif

	performingLogin = true;
	selfExecutingLogin = true;
	
	NSMutableDictionary* post_dict = [[NSMutableDictionary alloc] initWithCapacity:6];
	[post_dict setObject:SVZ_user forKey:@"email"];
	[post_dict setObject:SVZ_pass forKey:@"password"];
	[post_dict setObject:@"Einloggen" forKey:@"login"];
	[post_dict setObject:@"false" forKey:@"jsEnabled"];
	[post_dict setObject:theFormkey forKey:@"formkey"];
	[post_dict setObject:theIV forKey:@"iv"];
	
	NSString* regData = [post_dict formatForHTTP];

	NSURL *url = [NSURL URLWithString: [NSString stringWithFormat:VZ_LOGIN_PAGE, [SVZ_Account SVZLongAccountType]]];
	
	NSMutableURLRequest *post = [NSMutableURLRequest requestWithURL: url];
	
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): logging in, post target is: %@", SVZ_ProfileId, [url description]);
		NSString *len = [NSString stringWithFormat:@"%d", [regData length] ];
	//	AILogWithSignature( @"SVZProfileFetcher(%@): HTTBody for login is: %@, len: %@", SVZ_ProfileId, regData, len);
	#endif
	
	//now set the fields of the req
	[post setValue: @"application/x-www-form-urlencoded" forHTTPHeaderField: @"Content-Type"];
//	[post setValue: len forHTTPHeaderField: @"Content-Length"];
	NSString *ref = [NSString stringWithFormat: VZ_DEF_PAGE, [SVZ_Account SVZLongAccountType]];
	[post setValue: ref forHTTPHeaderField: @"Referer"];
	[post setValue: FF_USER_AGENT_STR forHTTPHeaderField: @"User-Agent"];
	[post setHTTPMethod: @"POST"];
	[post setHTTPBody: [regData dataUsingEncoding: NSASCIIStringEncoding]];
	[post setCachePolicy: NSURLRequestReloadIgnoringLocalAndRemoteCacheData];			//set the cache policy to bypass local and remote caches, AFAP
	[post setTimeoutInterval: 180];														//set timeout to 3minutes

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): Post Req. for login is: %@", SVZ_ProfileId, post);
	#endif


	if (connection != nil)
		[connection release];
		
	connection = [[NSURLConnection alloc] initWithRequest:post delegate:self];


	//and create the receiving buffer
	if (connection) {
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): Connection is: %@, creating receive buffer...", SVZ_ProfileId, connection);
		#endif
		if (receivedData != nil)
			[receivedData release];
			
		receivedData = [[[NSMutableData dataWithCapacity: 50000] retain] retain];
	}

	[post_dict release];

}

//attempts to fetch a profile for the id that this class intance represents
- (void)fetchProfile
{

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): fetching profile for: %@",SVZ_ProfileId, self);
	#endif
	

	@try{
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): creating req for: %@",SVZ_ProfileId, profile_url);
		#endif
		
		// @TODO check if there is a way to disable any caching on this connection, seems that quick refetch only gets a cached copy 
		NSURLRequest *theRequest=[NSURLRequest requestWithURL:[NSURL URLWithString: profile_url]
                        cachePolicy:NSURLRequestUseProtocolCachePolicy
						timeoutInterval:60.0];
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): req is: %@",SVZ_ProfileId, [theRequest description]);
		#endif

		if (connection != nil){
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): releasing old connection: %@",SVZ_ProfileId, [connection description]);
			#endif
			[connection release];
			
		}
		
		connection = [ [NSURLConnection alloc] initWithRequest:theRequest delegate:self];
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): new connection is: %@",SVZ_ProfileId, [connection description]);
			#endif

		if (receivedData != nil){
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): releasing old recvData",SVZ_ProfileId);
			#endif
			[receivedData release];
		}

		receivedData = [[ [NSMutableData dataWithCapacity: 50000] retain] retain];
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): new recvData is: %@",SVZ_ProfileId, [receivedData description]);
			#endif


	} @catch ( NSException *e ) {
			AILogWithSignature( @"SVZProfileFetcher(%@): Exception setting status %@: %@\n", SVZ_ProfileId, [e name], [e reason] );
	}
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): fetchProfile returning...",SVZ_ProfileId);
	#endif


}

//this fetches the actual pic from the profile, moved here to provide some actual data to the url req
- (NSImage *) fetchPicWithURL: (NSString *)theurl{


	NSURL *url = [NSURL URLWithString: theurl];
	
	NSMutableURLRequest *get = [NSMutableURLRequest requestWithURL: url];
	
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): fetching pic, url is: %@", SVZ_ProfileId, [url description]);
	#endif
	
	//now set the fields of the req
	[get setHTTPMethod: @"GET"];
	[get setValue: profile_url forHTTPHeaderField: @"Referer"];
	[get setValue: FF_USER_AGENT_STR forHTTPHeaderField: @"User-Agent"];
	[get setCachePolicy: NSURLRequestReloadIgnoringLocalAndRemoteCacheData];			//set the cache policy to bypass local and remote caches, AFAP
	[get setTimeoutInterval: 180];														//set timeout to 3minutes

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): Picfetch req'ing sync'ily: %@", SVZ_ProfileId, [get description]);
	#endif
		
	NSError *nse = nil;
	NSURLResponse *nsur = nil;
	
	
	NSData *data = [NSURLConnection sendSynchronousRequest:get returningResponse:&nsur error:&nse];

	#if DEBUG_BUILD
		//just refresh the inst. vars in case they got stale
		[self PreEventloopVarScopeCheck];
		
		if (data != nil){
			AILogWithSignature(@"SVZProfileFetcher(%@): Picfetch done fetching, received data.", SVZ_ProfileId);
		}else{
			AILogWithSignature( @"SVZProfileFetcher(%@): Picfetch done fetching, download failed!", SVZ_ProfileId);
		}
		
		if (nsur != nil)
			AILogWithSignature( @"SVZProfileFetcher(%@): Picfetch ret resp. is %@", SVZ_ProfileId, [nsur description]);
		if (nse != nil)
			AILogWithSignature( @"SVZProfileFetcher(%@): Picfetch ret error is %@", SVZ_ProfileId, [nse localizedDescription]);
	#endif


	NSImage *nsi = [[NSImage alloc] initWithData:data];

	return nsi;

}



/* ----------------------------------------- Delegates for the URLconnection events ---------------------------------------------------- */

#pragma mark delegates

- (void)connection:(NSURLConnection *)theConnection didReceiveResponse:(NSURLResponse *)response{ 

	[self PreEventloopVarScopeCheck];

	@try{
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): connection %@ successful with response: %@", SVZ_ProfileId, theConnection, response);
			AILogWithSignature( @"SVZProfileFetcher(): setting receivedData.length to 0");	//SVZ_ProfileId);
		#endif
		
		//just make sure that it is really a mutabledata instance
		if (![receivedData isKindOfClass:[NSMutableData class]])
			receivedData = [[[NSMutableData dataWithCapacity: 50000] retain] retain];
		

		if ([receivedData length] > 0)
			[receivedData setLength:0];
		
			
	} @catch ( NSException *e ) {
			AILogWithSignature( @"SVZProfileFetcher(%@): Exception setting receivedData to 0 %@: %@\n", SVZ_ProfileId, [e name], [e reason] );
	}
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): receivedData set to 0, and returning", SVZ_ProfileId);
	#endif
	
	[self PostEventloopVarRefcountCheck];

}

- (void)connection:(NSURLConnection *)theConnection didReceiveData:(NSData *)data{
	
	[self PreEventloopVarScopeCheck];
	
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): connection receivedData, appending %i bytes", SVZ_ProfileId, [data length]);
	#endif

	@try{
		[receivedData appendData:data];
	} @catch ( NSException *e ) {
			AILogWithSignature( @"SVZProfileFetcher(%@): Exception appending receivedData to buffer %@: %@\n",SVZ_ProfileId,  [e name], [e reason] );
	}

	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): connection receivedData returning", SVZ_ProfileId);
	#endif
	[self PostEventloopVarRefcountCheck];


}

- (void)connection:(NSURLConnection *)theConnection didFailWithError:(NSError *)error
{

	[self PreEventloopVarScopeCheck];
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): connection '%@' failed with error: '%@'", SVZ_ProfileId, theConnection, error);
	#endif
	
	//decide what to do
	if (selfExecutingLogin){
			#if	DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): login attempt #%i failed...", SVZ_ProfileId, numLoginAttempts);
			#endif
			numLoginAttempts++;
			
			if (numLoginAttempts < MAX_LOGINS_ATTEMPTS){
				#if	DEBUG_BUILD
					AILogWithSignature( @"SVZProfileFetcher(%@): proceeding to attempt #%d", SVZ_ProfileId, numLoginAttempts);
				#endif
				
				[self fetchProfile];
				[self PostEventloopVarRefcountCheck];

				return;
			}
	}else{
			#if	DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): fetch attempt #%i failed...", SVZ_ProfileId, numFetchAttempts);
			#endif
			numFetchAttempts++;
			
			if (numFetchAttempts < MAX_FETCH_ATTEMPTS){
				#if	DEBUG_BUILD
					AILogWithSignature( @"SVZProfileFetcher(%@): proceeding to attempt #%d", SVZ_ProfileId, numFetchAttempts);
				#endif
				[self fetchProfile];
				[self PostEventloopVarRefcountCheck];
				
				return;
			}
	}
	
	//max login/fetch attempts reached, abort and destroy
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): permanent error, aborting!", SVZ_ProfileId);
	#endif
	
	//notify all waiting observers, that they are to cease operation and destroy themselves (no point in coninuing)
	[StudiVZProfileFetcher notifyAllObserversToDestroy];
			
	//and destroy ourselves too
	[self destroy];

}

/* our connection will get forwarded to another adress as per 301 received from server, this gives us a chance to allow or abort */
-(NSURLRequest *)connection:(NSURLConnection *)theConnection
            willSendRequest:(NSURLRequest *)request
           redirectResponse:(NSURLResponse *)redirectResponse

{
	NSURLRequest * newrequest = request;
	[self PreEventloopVarScopeCheck];
	
    if (redirectResponse) {
	
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): connection %@ got response %@ to redirect to %@", SVZ_ProfileId, theConnection, redirectResponse, request);
		#endif
		
		//if we get forwarded to an URL ending w/ /Start, then we were sucessfully logged in, and since this never returns anything, dissallow redirect and just re-request profile
		//also if we get forwarded to to a new URL ending in profile/UID the login was sucessful
		if ([[[request URL] absoluteString] hasSuffix:@"Start"] ){
			//disallow by setting the answer to nil
			newrequest = nil;
			
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): disallowing redirect", SVZ_ProfileId);
				AILogWithSignature( @"SVZProfileFetcher(%@): login successfull!", SVZ_ProfileId);
			#endif

			//and notify all waiting objects that we are logged in now 
			[StudiVZProfileFetcher notifyAllObserversOfLogin];
			selfExecutingLogin = false;
			performingLogin = false;

			//and tell ourselves to refetch profile
			[self fetchProfile];

		}else if ([[[request URL] absoluteString] hasSuffix: [@"Profile/" stringByAppendingString:SVZ_ProfileId]]){
		
			//this means that we have sucessfully logged in and are being forwarded to the profile page already, so indicate we are logged in and let the forward go thru
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): login successfull!", SVZ_ProfileId);
				AILogWithSignature( @"SVZProfileFetcher(%@): being redirected to profile already", SVZ_ProfileId);
			#endif

			//and notify all waiting objects that we are logged in now 
			[StudiVZProfileFetcher notifyAllObserversOfLogin];
			selfExecutingLogin = false;
			performingLogin = false;

		
		}else if ( [[[request URL] absoluteString] hasSuffix: @"Captcha"] ){
			/* Uhhhhhh-oohhhh !!!!
			*  we have been making to many requests and are being forwarded to a captcha page
			*  seriously __BAAADDDD__ karma!
			*  short of using another account, there is nothing we can do
			*  #TODO see if that is possible somehow - seems its not, only way is to tell user to log in open someones profile page and do the captcha
			*  #TODO get the captcha, present it to user to solve and retry
			*  so kill all connections, tell all waiting observers to cease, release ourself and return, disallowing the redirect
			*/
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): profile fetch unsuccessfull! Moreover we are being forwarded to a captcha, seriously BAD news! Requesting User to solve!", SVZ_ProfileId);
			#endif

			//allow redirect to the captcha, to get it and solve it
			return request;

/*
			if (!UserNotifiedAboutCaptcha){
				//send an alert to alert the user
				[[NSAlert alertWithMessageText:AILocalizedString(@"StudiVZ",nil)
					defaultButton:AILocalizedString(@"OK",nil)
					alternateButton:nil
					otherButton:nil
					informativeTextWithFormat:[NSString stringWithFormat:
						AILocalizedString(@"Captcha Alert!\nThe SVZ-ProfileFetcher using account '%@' has been presented with a captcha, aborting!\nPlease log in to the website with this account, do the captcha and try again.",nil),
							SVZ_user]] 
					runModal];

				//notify all waiting observers, that they are to cease operation and destroy themselves (no point in coninuing)
				[StudiVZProfileFetcher notifyAllObserversToDestroy];
				UserNotifiedAboutCaptcha = true;
				
				//and destroy ourselves too
				[self destroy];
	
			}
			//and disallow redirect
			return nil;
	*/

		}
    }
	
	[self PostEventloopVarRefcountCheck];
	//in all other cases allow the redirect, by returning the new req as an answer
    return newrequest;
}




- (void)connectionDidFinishLoading:(NSURLConnection *)theConnection{ 
	    
	[self PreEventloopVarScopeCheck];
	#if DEBUG_BUILD
		AILogWithSignature( @"SVZProfileFetcher(%@): Download succeeded! Received %d bytes of data",SVZ_ProfileId, [receivedData length]);
	#endif

	NSString *receivedString = NULL;
	
	//if we got 0 bytes that was the redirect, just return and wait for more
	if (receivedData == nil ||  [receivedData length] == 0){
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): received nothing! returning...",SVZ_ProfileId);
		#endif
		[self PostEventloopVarRefcountCheck];

		return;
	}


	receivedString = [[NSString alloc] initWithData:receivedData encoding: NSUTF8StringEncoding];

	//check if the decoding failed
	if (receivedString == nil ||  [receivedString length] == 0){
		#if DEBUG_BUILD
			AILogWithSignature( @"SVZProfileFetcher(%@): decoding string failed! returning...",SVZ_ProfileId);
		#endif
		[self PostEventloopVarRefcountCheck];

		return;
	}

		
	#if	DEBUG_BUILD
		#if DBG_SHOW_FULL_REVC_HTML
			AILogWithSignature( @"SVZProfileFetcher(%@): received: %@", SVZ_ProfileId, receivedString);
		#endif
	#endif


	//check if we were logging in or making a request for the profile
	if (selfExecutingLogin){
	
		//and now check if we were successfully logged in by checking for the homepage mark
		if ([receivedString isMatchedByRegex: VZ_LOGGED_IN_REGEX]){
			//we were logged in, then re-request the profile
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): login successfull!", SVZ_ProfileId);
			#endif
			[self fetchProfile];

		}else{
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): login attempt#%d unsuccessfull :( ", SVZ_ProfileId, numLoginAttempts);
			#endif
			numLoginAttempts++;
			
			if (numLoginAttempts < MAX_LOGINS_ATTEMPTS){
				#if	DEBUG_BUILD
					AILogWithSignature( @"SVZProfileFetcher(%@): proceeding to attempt #%d", SVZ_ProfileId, numLoginAttempts);
				#endif
				[self fetchProfile];
			}
		}
		
	}else{
		//we were fetching the profile page, lets take a look at the result then, shall we?
		//check if we need to log in
		if ([receivedString isMatchedByRegex: VZ_NOT_LOGGED_IN_REGEX]){
			//do the regex magic
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): not logged in!", SVZ_ProfileId);
			#endif
			
			NSString *formKey = NULL;
			NSString *iv = NULL;
			
			[receivedString getCapturesWithRegexAndReferences: LOGIN_FORMKEY_REGEX, 
											@"$1", &formKey, 
											nil]; 

			[receivedString getCapturesWithRegexAndReferences: LOGIN_IV_REGEX, 
											@"$1", &iv, 
											nil]; 
			
			//we are not logged in, login now
			[self executeLoginWithFormKey: formKey andIV: iv];
			
		}else if ([receivedString isMatchedByRegex: VZ_CAPTCHA_REGEX]){

			//do the captcha
			[self solveCaptchaFromPage:receivedString];
			
		}else{
		
			//we do not need to log in, we got the profile, yeha!
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): profile fetch successfull!", SVZ_ProfileId);
			#endif			
			
			NSString *Alias = nil;
			NSString *PicURL = nil;
			NSString *RemoteGroup = [SVZ_Account SVZGroupName];
			NSString *net = [[SVZ_Account SVZAccountType] stringByAppendingString:@"vz"];
			NSString *RemoteStatus = nil;
			

			[receivedString getCapturesWithRegexAndReferences: PROFILE_ALIAS_REGEX, 
											@"$2", &Alias, 
											nil]; 

			[receivedString getCapturesWithRegexAndReferences: PROFILE_PIC_REGEX, 
											@"$1", &PicURL, 
											nil];
			
			[receivedString getCapturesWithRegexAndReferences: PROFILE_STATUS_REGEX,
											@"$2", &RemoteStatus,
											nil];


			if ([receivedString isMatchedByRegex: PROFILE_MVZ_REGEX]){
				RemoteGroup = [NSString stringWithString:@"MeinVZ"];
				net = [NSString stringWithString:@"mvz"];
			}else if ([receivedString isMatchedByRegex: PROFILE_SVZ_REGEX]){
				RemoteGroup = [NSString stringWithString:@"StudiVZ"];
				net = [NSString stringWithString:@"svz"];
			}
			
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): got Alias: %@", SVZ_ProfileId, Alias);
			#endif

			//set the display name now
			#if SET_NAME_FROM_PROFILE
				//#TODO check if there is an old alias somewhere which prevents the new one (provided contact has changed it) from being set
				if ( ![Alias isEqualToString:VZ_CAPTCHA_TITLE] 
					&& (	
						[SVZ_Contact displayName] == nil 
						|| [[SVZ_Contact displayName] isEqualToString: [SVZ_Contact formattedUID]]
						|| [[SVZ_Contact displayName] isEqualToString:VZ_CAPTCHA_TITLE]
						)
					){
					#if DEBUG_BUILD
						AILogWithSignature( @"SVZProfileFetcher(%@): setting Alias: %@", SVZ_ProfileId, Alias);
					#endif
					
					[SVZ_Contact setDisplayName:Alias];
				}
				
			#endif
			
			
			//set the group
			@try {
				#if DEBUG_BUILD
					AILogWithSignature( @"SVZProfileFetcher(%@): got Group: %@", SVZ_ProfileId, RemoteGroup);
				#endif
				
				if ([[SVZ_Account SVZGroupName] isEqualToString: RemoteGroup]){
					#if DEBUG_BUILD
						AILogWithSignature( @"SVZProfileFetcher(%@): removing group '%@' from account", SVZ_ProfileId, [SVZ_Account SVZGroupName]);
					#endif
					[SVZ_Contact removeRemoteGroupName: [SVZ_Account SVZGroupName]];
				
					#if DEBUG_BUILD
						AILogWithSignature( @"SVZProfileFetcher(%@): adding group '%@' to account", SVZ_ProfileId, RemoteGroup);
					#endif
					[SVZ_Contact addRemoteGroupName:RemoteGroup];
					
				}
					
				#if DEBUG_BUILD
					AILogWithSignature( @"SVZProfileFetcher(%@): got Status: %@", SVZ_ProfileId, RemoteStatus);
				#endif
				
				//and the status
				if (RemoteStatus != nil){
					#if DEBUG_BUILD
						AILogWithSignature( @"SVZProfileFetcher(%@): setting Status to: %@", SVZ_ProfileId, RemoteStatus);
					#endif
					
					[SVZ_Contact setStatusMessage:[[[NSAttributedString alloc] initWithString:RemoteStatus] autorelease] notify:NotifyLater];
				}
			} @catch ( NSException *e ) {
				AILogWithSignature( @"SVZProfileFetcher(%@): Exception setting status %@: %@\n", SVZ_ProfileId, [e name], [e reason] );
			}

			//and lastly download the pic
			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): got Pic URL: %@", SVZ_ProfileId, PicURL);
			#endif			
			
			
			NSImage *image = nil;
			
			if ([PicURL hasSuffix:VZ_NOPIC]){
				//do nothing in this case, coz it will fall through to the service icon if needed
			}else{
				//try getting the image
				@try{
					#if DEBUG_BUILD
						AILogWithSignature( @"SVZProfileFetcher(%@): requesting pic from URL: %@", SVZ_ProfileId, PicURL);
					#endif			

					//image = [[NSImage alloc] initWithContentsOfURL: [NSURL URLWithString: PicURL] ];
					image = [self fetchPicWithURL: PicURL];
					
				} @catch ( NSException *e ) {
					AILogWithSignature( @"SVZProfileFetcher(%@): Exception fetching profile pic %@: %@\n", SVZ_ProfileId, [e name] , [e reason]  );
				}
			}
			
			#if EXPL_SET_BUD_SRV_PIC_IF_DIF
			if (image == nil || ![image isValid]){
				NSString *imPref = [SVZ_Account SVZAccountType];
				
				NSString *imageName = [imPref stringByAppendingString:@"vz"];

				if (![net isEqualToString: imageName])
					image =	[NSImage imageNamed:imageName forClass:[self class]];
				
				#if DEBUG_BUILD
					AILogWithSignature( @"SVZProfileFetcher(%@): falling back to service Icon...", SVZ_ProfileId);
				#endif
				
			}
			#endif
			
			if (image != nil && [image isValid]){
				@try {
					
					#if DEBUG_BUILD
						AILogWithSignature( @"SVZProfileFetcher(%@): setting account pic...", SVZ_ProfileId);
					#endif
					
					//set it
					NSData *userIconData = [image TIFFRepresentation];
					if ([userIconData length] > 10){
						[SVZ_Contact setServersideIconData:userIconData notify:NotifyLater];
					}
							
				} @catch ( NSException *e ) {
					AILogWithSignature( @"SVZProfileFetcher(%@): Exception setting profile pic %@: %@\n", SVZ_ProfileId, [e name], [e reason] );
				}
			}

			#if DEBUG_BUILD
				AILogWithSignature( @"SVZProfileFetcher(%@): notifying buddy of update...", SVZ_ProfileId);
			#endif
			
			//and notify contact list of change
			[SVZ_Contact notifyOfChangedPropertiesSilently:YES];
			
			//and destroy ourselves
			[self destroy];
			return;
		}
	}
	[self PostEventloopVarRefcountCheck];

}


@end
